package pp.s1184725.boppi;

import java.util.*;
import java.util.stream.Collectors;

import pp.iloc.eval.Machine;

public class FunctionType implements Type {
	private List<Type> parameters;
	private List<Integer> offsets;
	private Type result;
	private int localDataSize;
	private int depth;

	public FunctionType(Type returnType, List<Type> parameterTypes, int depth) {
		result = returnType;
		parameters = new ArrayList<>(parameterTypes);
		offsets = new ArrayList<>();
		this.depth = depth;
	}

	@Override
	public int getSize() {
		return Machine.INT_SIZE;
	}

	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof FunctionType))
			return false;

		FunctionType other = (FunctionType) obj;
		return result.equals(other.result) && parameters.equals(other.parameters);
	}

	@Override
	public String toString() {
		return parameters.stream().map(Type::toString).collect(Collectors.joining(",", "(", ")")) + "->"
				+ result.toString();
	}

	/**
	 * Sets the size (in bytes) of the local data segment for this function.
	 * 
	 * @param newSize
	 *            the size (in bytes) of the local data segment
	 */
	public void setLocalDataSize(int newSize) {
		localDataSize = newSize;
	}

	/**
	 * Returns the size (in bytes) of the local data segment for this function.
	 * 
	 * @return the size (in bytes) of the local data segment
	 */
	public int getLocalDataSize() {
		return localDataSize;
	}

	/**
	 * Returns the result type of this function.
	 * 
	 * @return the result type
	 */
	public Type getReturn() {
		return result;
	}

	/**
	 * Returns the parameter types of this function.
	 * 
	 * @return the parameter types
	 */
	public List<Type> getParameters() {
		return parameters;
	}

	public void setParameters(List<Type> parameters) {
		this.parameters = parameters;
	}

	public List<Integer> getOffsets() {
		return offsets;
	}

	public int getDepth() {
		return depth;
	}

	public void setDepth(int newDepth) {
		depth = newDepth;
	}
}
