package pp.s1184725.boppi.test;

import java.util.List;
import java.util.logging.*;

import org.antlr.v4.runtime.CharStream;
import org.junit.Test;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;
import pp.s1184725.boppi.*;

public class ParserTest {
	private List<LogRecord> log;

	private void parseString(String code) {
		parse(ToolChain.getCharStream(code));
	}

	private void parseFile(String file) {
		parse(ToolChain.getCharStream(AllTests.directory.resolve(file)));
	}

	private void parse(CharStream stream) {
		Logger logger = Logger.getAnonymousLogger();
		log = ToolChain.makeListLog(logger);
		ToolChain.getParser(ToolChain.getLexer(stream, logger), logger).program();
	}

	@Test
	public void correctExpressionTest() {
		parseFile("simpleExpression.boppi");
		assertThat(log, empty());
	}

	@Test
	public void wrongExpressionTest() {
		parseString("");
		assertThat(log, not(empty()));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		parseString("~");
		assertThat(log, not(empty()));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		parseString("0A");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		parseString("do");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		parseString("true true");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));
	}

	@Test
	public void correctVariableTest() {
		parseFile("simpleVariable.boppi");
		assertThat(log, empty());
	}

	@Test
	public void wrongVariableTest() {
		parseString("var");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		parseString("var bool 5");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		parseString("var 'c' varname");
		assertThat(log, not(empty()));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		parseString("var bool; true true;");
		assertThat(log, hasSize(2));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));
	}

	@Test
	public void correctScopeTest() {
		parseFile("simpleScope.boppi");
		assertThat(log, empty());
	}

	@Test
	public void correctConditionalTest() {
		parseFile("if.boppi");
		assertThat(log, empty());

		parseFile("while.boppi");
		assertThat(log, empty());
	}

}
